; ********************************************* ;
pro scattering_function,x,p,yfit,_Extra = extra
bg = p[0] & area_el = p[1] & cen_el = p[2]
area_iel = p[3] & cen_iel = p[4] & fwhm_iel = p[5]
y_iel = lorentzian(x,area = area_iel,cen = cen_iel,fwhm = fwhm_iel)
nlo = 3 & nhi = n_elements(x) - nlo - 1
dx = x[1] - x[0]
con = convol(*extra.res_ptr,reverse(y_iel[nlo:nhi]),/edge_truncate) * dx
yfit = bg+gaussian(x,area = area_el,cen = cen_el,fwhm = 1.0)+con
end
; ********************************************* ;
pro my_iterproc,  fnc,p,iter,fnorm,    $
                  dof = dof,           $
                  quiet = quiet,       $
                  parinfo = parinfo,   $
                  _Extra = extra
; Routine for performing a custom update procedure
scattering_function,extra.x,p,yfit, $
   _Extra = {res_ptr:extra.res_ptr}
wset,extra.winpix
plot,extra.x,extra.y,psym = 4
errplot,extra.x,extra.y-extra.dy,extra.y+extra.dy,width = 0.0
oplot,extra.x,yfit,psym = 0,thick = 2.0
wset,extra.winvis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,extra.winpix]

end
; ********************************************* ;
pro analyze_fake_data_c
; First read in the data
filename = sourceroot()+'fake_data.txt'
count = 0L
dum1 = (dum2 = (dum3 = 0.0))
openr,lun,filename,/get_lun
while not eof(lun) do begin
   readf,lun,dum1,dum2,dum3
   if count eq 0L then begin
      x = dum1 & y = dum2 & dy = dum3
   endif else begin
      x = [x,dum1] & y = [y,dum2] & dy = [dy,dum3]
   endelse
   count++
endwhile
free_lun,lun,/force

; Now plot the data with the error bars
winvis = 0
window,0,xsize = 400,ysize = 400
window,/free,/pixmap,xsize = 400,ysize = 400
winpix = !d.window
wset,winpix
plot,x,y,psym = 4
errplot,x,y-dy,y+dy,width = 0.0
wset,winvis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,winpix]

; We will assume that we know that the resolution
; function is a Gaussian with a FWHM of 1.0 and is
; defined over the same interval as the data.
res = gaussian(x,area = 1.0,center = 0.0, fwhm = 1.0)
; Create a pointer to the resolution function because we
; will not want to pass around the vector and keep making
; copies of it.  A single reference to a heap variable
; is all that's necessary.
res_ptr = ptr_new(res)

; Pick some initial guesses for the fit
bg = 10.0 & area_el = 500.0 & cen_el = 0.0
area_iel = 100.0 & cen_iel = 5.0 & fwhm_iel = 1.0
p = [bg,area_el,cen_el,area_iel,cen_iel,fwhm_iel]
; Impose constraints on each of the positive-definite
; parameters that they be greater than zero.
parinfo = replicate({limited:[0,0],limits:[0D,0D]},n_elements(p))
parinfo[0].limited[0] = 1
parinfo[0].limits[0] = 0.D
parinfo[1].limited[0] = 1
parinfo[1].limits[0] = 0.D
parinfo[3].limited[0] = 1
parinfo[3].limits[0] = 0.D
parinfo[5].limited[0] = 1
parinfo[5].limits[0] = 0.D

iterargs = {x:x,y:y,dy:dy,winpix:winpix,winvis:winvis,res_ptr:res_ptr}
p = double(p)
functargs = {res_ptr:res_ptr}
yfit = mpcurvefit(x,y,1d/dy^2,p, sigma, parinfo = parinfo, $
   function_name = 'scattering_function', $
   /noderivative,functargs = functargs, $
   iterargs = iterargs,iterproc = 'my_iterproc')

wdelete,winpix

;oplot,x,yfit,psym = 0,thick = 2.0
for i = 0,n_elements(p)-1 do $
   print,strtrim(string(p[i]),2)+' +/- '+strtrim(string(sigma[i]),2)

; Use Monte-Carlo uncertainty estimation for the fit
; parameters.

; First create the synthetic data
nmc = 300
nx = n_elements(x)
data = rebin(y,nx,nmc,/sample) + $
   (rebin(dy,nx,nmc,/sample)) * randomn(s,nx,nmc)
nparams = n_elements(p)
mc_params = fltarr(nparams,nmc)
; Now loop through NMC times and gather the statistics
for i = 0,nmc-1 do begin
   yfit = mpcurvefit(x,data[*,i],1d/dy^2,p, sigma, $
      parinfo = parinfo, $
      function_name = 'scattering_function', $
      /noderivative,functargs = functargs,/quiet)
   mc_params[*,i] = p
endfor
; Calculate the statistics on the Monte-Carlo estimates
sigma_p = fltarr(nparams)
for i = 0,nparams-1 do begin
   sigma_p[i] = sqrt((moment(mc_params[i,*]))[1])
endfor
print
print
for i = 0,nparams-1 do $
   print,strtrim(string(sigma[i]/p[i]),2)+ $
   ' , '+strtrim(string(sigma_p[i]/p[i]),2)

ptr_free,res_ptr
end