pro irreg_convol_example
; Create an irregular grid of values for the independent
; variable for the "resolution function"
xrlo = -3.0 & xrhi = 1.0 & nr = 50
xres = xrlo+(xrhi-xrlo)*randomu(s,nr)
; Sort the values since adding the random component
; might shift the order of the points.
xres = xres[sort(xres)]

; Create the irregular grid of values for the independent
; variable for the "intrinsic function"
nx = 100 & xlo = -3.0 & xhi = 15.0
x = xlo+(xhi-xlo)*randomu(s,nx)
; Sort these ones for the same reason that we
; sorted the resolution points.
x = x[sort(x)]

cen = 2.0 & cenr = 0.0
fwhm = 2.0 & fwhmr = 1.0
area = 2.0 & arear = 1.0

; Create the resolution function
res = gaussian(xr,area = arear,cen = cenr,fwhm = fwhmr)
; Create the intrinsic function
y = gaussian(x,area = area,cen = cen,fwhm = fwhm)

; Use the irreg_convol function to perform the
; convolution approximation.
con = irreg_convol(xr,res,x,y)

plot,x,con,psym = 4,xrange = [-5,7],/xstyle,symsize = 2

; Calculate and plot the theoretical convolution result
xth = makepts(xlo = -5.0,xhi = 5.0,npts = 200)
yth = gaussian(xth,  area = area*arear,center = cen+cenr,   $
                     fwhm = sqrt(fwhm^2+fwhmr^2))
oplot,xth,yth,psym = 0,thick = 2.0

print,'Calculated integrated intensity: ',int_tabulated(x,con)
print,'Area (theory): ',area*arear

; Write the jpeg
tvlct,r,g,b,/get
image2d = tvrd()
s = size(image2d,/dimensions)
image24 = bytarr(3,s[0],s[1])
tvlct,r,g,b,/get
image24[0,*,*] = 255b - (r[image2d])
image24[1,*,*] = 255b - (g[image2d])
image24[2,*,*] = 255b - (b[image2d])

filename = 'c:\idl_training\ornl\documentation\irreg_convol.jpg'
write_jpeg,filename,image24,quality = 100,true = 1

end