; MAKE_FAKE_DATA.PRO
;
; Creates three vectors: x,y,yerr
; x: independent variable
; y: dependent variable ("data")
; yerr: error in y
;
; Keywords:
;  DRAW:  if set, the data with errors will be plotted to screen.
;  WRITE: if set, 3-column ascii file will be created.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro make_fake_data,x,y,yerr,draw = draw,write = write
!except = 0 ; suppress mathematical underflow messages
npts = 200

; Create the grid of values for the independent variable
x = makepts(xlo = -10.0, xhi = 10.0, npts = npts)

; Create a resolution function: a Gaussian with FWHM of 1.0
res = gaussian(x,area = 1.0,cen = 0.0,fwhm = 1.0)

; Create the elastic component: a Gaussian with area 600.0
y1 = 600.0*res
y2 = lorentzian(x,area = 300,cen = 5.0,fwhm = 1.0)

; Convolve the Lorentzian with the resolution function
dx = x[1]-x[0]
nlo = 3 & nhi = npts-nlo+1
y2con = convol(y2,reverse(res[nlo:nhi]),/edge_truncate)*dx

y = fltarr(npts)
yerr = fltarr(npts)
for i = 0,npts-1 do begin
  y[i] = randomn(s,1,poisson = y1[i]+y2con[i]+20.0)
endfor

yerr = sqrt(y)
if keyword_set(draw) then begin
  plot,x,y,psym = 4, xtitle = '!4x!3', $
         ytitle = '!3S(!4x!3)',        $
         title = 'Measured Intensity'
  oplot,x,y,psym = 4
  dx = x[1]-x[0]
  errplot,x,y-yerr,y+yerr,width = 0.0

endif

if keyword_set(write) then begin
   cd,current = dir
   filename = dir+path_sep()+'fake_data.txt'
   openw,lun,filename,/get_lun
   for i = 0,npts-1 do begin
      printf,lun,x[i],y[i],yerr[i],format = '(3f15.6)'
   endfor
   free_lun,lun,/force
endif

end